@@ -27,6 +27,7 @@
         
 ;; Feature masks:
 (define droppable-feature 1)
+(define scores-disabled 2)
 
 (define (reset-features) (set-feature-word! 0))
 
@@ -283,6 +284,11 @@
 (define (get-value card)
       (car card))
 
+;; WARNING: This generates a synthetic card that isn't part of the game.
+;;          See gaps.scm for an example of its intended use.
+(define (add-to-value card n)
+  (cons (+ (car card) n) (cdr card)))
+
 ; slot procedures
 (define (get-cards slot-id)
   (cadr (get-slot slot-id)))
@@ -612,7 +618,7 @@
   (lambda (names)
     (if (equal? '() names)
         '()
-        (cons (eval-1 (car names)) 
+        (cons (eval-1 (list 'copy-tree (car names))) 
               (save-variables (cdr names))))))
 
 ; Restore all the state variables for a game
@@ -620,7 +626,7 @@
   (lambda (names values)
     (or (equal? '() names)
         (begin
-          (eval-1 (list 'set! (car names) (car values)))
+          (eval-1 (list 'set! (car names) (list 'quote (car values))))
           (restore-variables (cdr names) (cdr values))
           ))))
 
