@@ -21,32 +21,23 @@
 #include "blockops.h"
 #include "blocks.h"
 #include "field.h"
-#include "gnome-canvas-pimage.h"
 
-GnomeCanvasItem *BlockOps::generateItem(int x, int y, int colour)
-{
-	GnomeCanvasItem *	it = gnome_canvas_item_new(
-  			gnome_canvas_root(GNOME_CANVAS(fieldDisplay->getWidget())),
-  			gnome_canvas_pimage_get_type(),
-  			"image", pic[colour],
-  			"x", (double) x * BLOCK_SIZE,
-  			"y", (double) y * BLOCK_SIZE,
-			"width", (double) BLOCK_SIZE,
-			"height", (double) BLOCK_SIZE,
-  			NULL);
+#define NCOLOURS 7
 
-	return it;
-}
 
-BlockOps::BlockOps(Field *f)
+BlockOps::BlockOps() :
+  blocknr (0),
+  rot (0),
+  color (0)
 {
 	field = new Block*[COLUMNS];
-	fieldDisplay = f;
 	
+	posx = COLUMNS / 2;
+	posy = 0;
+
 	for (int i = 0; i < COLUMNS; ++i)
 		field[i] = new Block[LINES];
 
-	field_initialized = false;
 	emptyField();
 }
 
@@ -62,7 +53,7 @@ bool
 BlockOps::blockOkHere(int x, int y, int b, int r)
 {
 	x -= 2;
-	
+
 	for (int x1 = 0; x1 < 4; ++x1)
 	{
 		for (int y1 = 0; y1 < 4; ++y1)
@@ -77,6 +68,7 @@ BlockOps::blockOkHere(int x, int y, int b, int r)
 				return false;
 		}
 	}
+
 	return true;	
 }
 
@@ -102,6 +94,7 @@ BlockOps::getLinesToBottom()
 				lines = tmp;
 		}
 	}
+
 	return lines;
 }
 
@@ -203,57 +196,55 @@ BlockOps::fallingToLaying()
 }
 
 void
-BlockOps::checkFullLines(ScoreFrame *s)
+BlockOps::eliminateLine(int l)
 {
-	bool found;
-	int numFullLines = 0;
-	
-	do
+	for (int y = l; y > 0; --y)
 	{
-		found = false;
-		
-		for (int y = LINES - 1; y >= 0; --y)
+		for (int x = 0; x < COLUMNS; ++x)
 		{
-			bool f = true;
-			for (int x = 0; x < COLUMNS; ++x)
+			field[x][y] = field[x][y - 1];
+
+			field[x][y - 1].what = EMPTY;
+			field[x][y - 1].color = 0;
+		}
+	}
+}
+
+int
+BlockOps::checkFullLines()
+{
+	// we can have at most 4 full lines (vertical block)
+	int fullLines[4] = {0, };
+	int numFullLines = 0;
+
+	for (int y = posy; y < MIN(posy + 4, LINES); ++y)
+	{
+		bool f = true;
+		for (int x = 0; x < COLUMNS; ++x)
+		{
+			if (field[x][y].what != LAYING)
 			{
-				if (field[x][y].what != LAYING)
-				{
-					f = false;
-					break;
-				}
-			}
-			if (f)
-			{
-				++numFullLines;
-				s->checkLevel();
-				
-				found = true;
-				for (int y1 = y; y1 >= 0; --y1)
-				{
-					for (int x = 0; x < COLUMNS; ++x)
-					{
-						if (y1 < y)
-							field[x][y1 + 1] = field[x][y1];
-						field[x][y1].what = EMPTY;
-						if (field[x][y1].item)
-						{
-							gtk_object_destroy(GTK_OBJECT(field[x][y1].item));
-							field[x][y1].item = 0;
-							if (y1 < y)
-								field[x][y1 + 1].item = 
-									generateItem(x, y1 + 1, field[x][y1 + 1].color);
-						}
-						
-					}
-				}
+				f = false;
 				break;
 			}
 		}
+
+		if (f)
+		{
+			fullLines[numFullLines] = y;
+			++numFullLines;
+		}
 	}
-	while (found);
-	if(numFullLines > 0)
-		s->incLines(numFullLines);
+
+	if (numFullLines > 0)
+	{
+		for (int i = 0; i < numFullLines; ++i)
+		{
+			eliminateLine(fullLines[i]);
+		}
+	}
+
+	return numFullLines;
 }
 
 bool
@@ -262,15 +253,17 @@ BlockOps::generateFallingBlock()
 	posx = COLUMNS / 2 + 1;
 	posy = 0;
 
-	blocknr = blocknr_next == -1 ? rand() % tableSize : blocknr_next;
-	rot = rot_next == -1 ? rand() % 4 : rot_next;
-	int cn = random_block_colors ? rand() % nr_of_colors : blocknr % nr_of_colors;
+	blocknr = blocknr_next == -1 ? g_random_int_range(0, tableSize) :
+		blocknr_next;
+	rot = rot_next == -1 ? g_random_int_range(0, 4) : rot_next;
+	int cn = random_block_colors ? g_random_int_range(0, NCOLOURS) :
+		blocknr % NCOLOURS;
 	color = color_next == -1 ? cn : color_next;
 	
-	blocknr_next = rand() % tableSize;
-	rot_next = rand() % 4;
-	color_next = random_block_colors ? rand() % nr_of_colors : 
-		blocknr_next % nr_of_colors;
+	blocknr_next = g_random_int_range(0, tableSize);
+	rot_next = g_random_int_range(0, 4);
+	color_next = random_block_colors ? g_random_int_range(0, NCOLOURS) :
+		blocknr_next % NCOLOURS;
 	
 	if (!blockOkHere(posx, posy, blocknr, rot))
 		return false;
@@ -285,31 +278,22 @@ BlockOps::emptyField(int filled_lines, int fill_prob)
 
 	for (int y = 0; y < LINES; ++y)
 	{
-		blank = rand() % COLUMNS; // Allow for at least one blank per line
+ 		// Allow for at least one blank per line
+		blank = g_random_int_range(0, COLUMNS);
+
 		for (int x = 0; x < COLUMNS; ++x)
 		{
-			if (field_initialized && field[x][y].item != 0)
-			{
-				gtk_object_destroy(GTK_OBJECT(field[x][y].item));
-			}
-			field[x][y].item = 0;
 			field[x][y].what = EMPTY;
+			field[x][y].color = 0; 
+
 			if ((y>=(LINES - filled_lines)) && (x != blank) &&
-			    ((rand() % 10) < fill_prob)) { 
+			    ((g_random_int_range(0, 10)) < fill_prob)) { 
 				field[x][y].what = LAYING;
-				if (nr_of_colors)
-					field[x][y].color = rand() % nr_of_colors;
-				else
-					// This is in case we're called
-					// before the widgets are set 
-					// up and nr_of_colors is
-					// defined.
-					field[x][y].color = 0; 
-				field[x][y].item = generateItem(x,y,field[x][y].color);
+				field[x][y].color = 
+				  g_random_int_range(0, NCOLOURS);
 			}
 		}
 	}
-	field_initialized = true;
 }
 
 void
@@ -327,26 +311,18 @@ BlockOps::putBlockInField(bool erase)
 		{
 			if (blockTable[blocknr][rot][x][y])
 			{
-				field[posx - 2 + x][y + posy].what = erase ? EMPTY : FALLING;
 				if (erase)
 				{
-					GnomeCanvasItem *o = field[posx - 2 + x][y + posy].item;
-					if (o)
-						gtk_object_destroy(GTK_OBJECT(o));
-					field[posx - 2 + x][y + posy].item = 0;
+					field[posx - 2 + x][y + posy].what = EMPTY;
+					field[posx - 2 + x][y + posy].color = 0;
 				}
 				else
 				{
+					field[posx - 2 + x][y + posy].what = FALLING;
 					field[posx - 2 + x][y + posy].color = color;
-					field[posx - 2 + x][y + posy].item = 
-						generateItem(posx - 2 + x, y + posy, 
-									 field[posx - 2 + x][y + posy].color);
 				}
 			}
 		}
 	}
 }
 
-
-
-
