@@ -27,10 +27,13 @@
 #include "preview.h"
 #include "scoreframe.h"
 #include "sound.h"
+#include "highscores.h"
+#include "renderer.h"
 
 #include <games-gconf.h>
 #include <games-frame.h>
 #include <games-controls.h>
+#include <games-stock.h>
 
 #include <libgnomevfs/gnome-vfs.h>
 
@@ -39,20 +42,11 @@
 #include <dirent.h>
 #include <string.h>
 
-GdkPixbuf **pic;
-
 int LINES = 20;
 int COLUMNS = 11;
 
 int BLOCK_SIZE = 40;
 
-int posx = COLUMNS / 2;
-int posy = 0;
-
-int blocknr = 0;
-int rot = 0;
-int color = 0;
-
 int blocknr_next = -1;
 int rot_next = -1;
 int color_next = -1;
@@ -63,26 +57,33 @@ bool rotateCounterClockWise = true;
 
 #define TETRIS_OBJECT "gnometris-tetris-object"
 
-#define KEY_OPTIONS_DIR "/apps/gnometris/options"
-#define KEY_SOUND "/apps/gnometris/options/sound"
-#define KEY_BLOCK_PIXMAP "/apps/gnometris/options/block_pixmap"
-#define KEY_STARTING_LEVEL "/apps/gnometris/options/starting_level"
-#define KEY_DO_PREVIEW "/apps/gnometris/options/do_preview"
-#define KEY_RANDOM_BLOCK_COLORS "/apps/gnometris/options/random_block_colors"
-#define KEY_ROTATE_COUNTER_CLOCKWISE "/apps/gnometris/options/rotate_counter_clock_wise"
-#define KEY_LINE_FILL_HEIGHT "/apps/gnometris/options/line_fill_height"
-#define KEY_LINE_FILL_PROBABILITY "/apps/gnometris/options/line_fill_probability"
+#define KEY_BASE "/apps/gnometris"
 
-#define KEY_CONTROLS_DIR "/apps/gnometris/controls"
-#define KEY_MOVE_LEFT "/apps/gnometris/controls/key_left"
-#define KEY_MOVE_RIGHT "/apps/gnometris/controls/key_right"
-#define KEY_MOVE_DOWN "/apps/gnometris/controls/key_down"
-#define KEY_MOVE_DROP "/apps/gnometris/controls/key_drop"
-#define KEY_MOVE_ROTATE "/apps/gnometris/controls/key_rotate"
-#define KEY_MOVE_PAUSE "/apps/gnometris/controls/key_pause"
+#define KEY_OPTIONS_DIR KEY_BASE "/options"
 
-#define KEY_BG_COLOUR "/apps/gnometris/options/bgcolor"
-#define KEY_USE_BG_IMAGE "/apps/gnometris/options/usebgimage"
+#define KEY_SOUND KEY_OPTIONS_DIR "/sound"
+#define KEY_BLOCK_PIXMAP KEY_OPTIONS_DIR "/block_pixmap"
+#define KEY_THEME KEY_OPTIONS_DIR "/theme"
+#define KEY_STARTING_LEVEL KEY_OPTIONS_DIR "/starting_level"
+#define KEY_DO_PREVIEW KEY_OPTIONS_DIR "/do_preview"
+#define KEY_RANDOM_BLOCK_COLORS KEY_OPTIONS_DIR "/random_block_colors"
+#define KEY_ROTATE_COUNTER_CLOCKWISE KEY_OPTIONS_DIR "/rotate_counter_clock_wise"
+#define KEY_LINE_FILL_HEIGHT KEY_OPTIONS_DIR "/line_fill_height"
+#define KEY_LINE_FILL_PROBABILITY KEY_OPTIONS_DIR "/line_fill_probability"
+
+#define KEY_CONTROLS_DIR KEY_BASE "/controls"
+#define KEY_MOVE_LEFT KEY_CONTROLS_DIR "/key_left"
+#define KEY_MOVE_RIGHT KEY_CONTROLS_DIR "/key_right"
+#define KEY_MOVE_DOWN KEY_CONTROLS_DIR "/key_down"
+#define KEY_MOVE_DROP KEY_CONTROLS_DIR "/key_drop"
+#define KEY_MOVE_ROTATE KEY_CONTROLS_DIR "/key_rotate"
+#define KEY_MOVE_PAUSE KEY_CONTROLS_DIR "/key_pause"
+
+#define KEY_BG_COLOUR KEY_OPTIONS_DIR "/bgcolor"
+#define KEY_USE_BG_IMAGE KEY_OPTIONS_DIR "/usebgimage"
+
+#define KEY_HEIGHT KEY_OPTIONS_DIR "/height"
+#define KEY_WIDTH KEY_OPTIONS_DIR "/width"
 
 #define TILE_THRESHOLD 65
 
@@ -95,38 +96,75 @@ enum {
 
 
 Tetris::Tetris(int cmdlLevel): 
-	blockPixmap(0),
+	themeno (0),
 	field(0),
-	preview(0),
 	paused(false), 
 	timeoutId(0), 
 	onePause(false), 
 	inPlay(false),
-	image(0),
 	bgimage(0),
 	setupdialog(0), 
 	cmdlineLevel(cmdlLevel), 
 	fastFall(false),
         dropBlock(false)
 {
-	double x1, y1, x2, y2;
-	double width;
-	double pts;
-	gchar * outdir;
-	GtkTargetEntry targets[] = {{"text/uri-list", 0, URI_LIST}, 
-				    {"property/bgimage", 0, URI_LIST},
-				    {"text/plain", 0, TEXT_PLAIN},
-				    {"STRING", 0, TEXT_PLAIN},
-				    {"application/x-color", 0, COLOUR},
-				    {"x-special/gnome-reset-background", 0, RESET}};
+	GtkUIManager *ui_manager;
+	GtkAccelGroup *accel_group;
+	GtkActionGroup *action_group;
+	GtkWidget *vbox;
+	GtkWidget *aspect_frame;
+	GtkWidget *menubar;
+	gint width, height;
 
+	gchar *outdir;
+	const GtkTargetEntry targets[] = {{"text/uri-list", 0, URI_LIST}, 
+					  {"property/bgimage", 0, URI_LIST},
+					  {"text/plain", 0, TEXT_PLAIN},
+					  {"STRING", 0, TEXT_PLAIN},
+					  {"application/x-color", 0, COLOUR},
+					  {"x-special/gnome-reset-background", 0, RESET}};
+
+	const GtkActionEntry actions[] = {
+	{ "GameMenu", NULL, N_("_Game") },
+	{ "SettingsMenu", NULL, N_("_Settings") },
+	{ "HelpMenu", NULL, N_("_Help") },
+	{ "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, G_CALLBACK (gameNew) },
+	{ "Pause", GAMES_STOCK_PAUSE_GAME, NULL, NULL, NULL, G_CALLBACK (gamePause) },
+	{ "Resume", GAMES_STOCK_RESUME_GAME, NULL, NULL, NULL, G_CALLBACK (gamePause) },
+	{ "Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL, G_CALLBACK (gameTopTen) },
+	{ "EndGame", GAMES_STOCK_END_GAME, NULL, NULL, NULL, G_CALLBACK (gameEnd) },
+	{ "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (gameQuit) },
+	{ "Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL, G_CALLBACK (gameProperties) },
+	{ "Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL, G_CALLBACK (gameHelp) },
+	{ "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (gameAbout) }
+	};
+
+	const char ui_description[] =
+	"<ui>"
+	"  <menubar name='MainMenu'>"
+	"    <menu action='GameMenu'>"
+	"      <menuitem action='NewGame'/>"
+	"      <menuitem action='Pause'/>"
+	"      <menuitem action='Resume'/>"
+	"      <separator/>"
+	"      <menuitem action='Scores'/>"
+	"      <menuitem action='EndGame'/>"
+	"      <separator/>"
+	"      <menuitem action='Quit'/>"
+	"    </menu>"
+	"    <menu action='SettingsMenu'>"
+	"      <menuitem action='Preferences'/>"
+	"    </menu>"
+	"    <menu action='HelpMenu'>"
+	"      <menuitem action='Contents'/>"
+	"      <menuitem action='About'/>"
+	"    </menu>"
+	"  </menubar>"
+	"</ui>";
+	
 	if (!sound)
 		sound = new Sound();
 
-	pic = new GdkPixbuf*[tableSize];
-	for (int i = 0; i < tableSize; ++i)
-		pic[i] = 0;
-
 	/* Locate our background image. */
 	outdir = g_build_filename (gnome_user_dir_get (), "gnometris.d", 
 				   NULL);
@@ -137,56 +175,21 @@ Tetris::Tetris(int cmdlLevel):
 	
 	w = gnome_app_new("gnometris", _("Gnometris"));
 	g_signal_connect (w, "delete_event", G_CALLBACK (gameQuit), this);
-	gtk_window_set_resizable (GTK_WINDOW (w), FALSE);
 	gtk_drag_dest_set (w, GTK_DEST_DEFAULT_ALL, targets, 
 			   G_N_ELEMENTS(targets), 
 			   GDK_ACTION_MOVE);
 	g_signal_connect (G_OBJECT (w), "drag_data_received", 
 			  G_CALLBACK (dragDrop), this);
-
-	static GnomeUIInfo game_menu[] = 
-	{
-		GNOMEUIINFO_MENU_NEW_GAME_ITEM(gameNew, this),
-		GNOMEUIINFO_MENU_PAUSE_GAME_ITEM(gamePause, this),
-		GNOMEUIINFO_SEPARATOR,
-		GNOMEUIINFO_MENU_SCORES_ITEM(gameTopTen, this),
-		GNOMEUIINFO_MENU_END_GAME_ITEM(gameEnd, this),
-		GNOMEUIINFO_SEPARATOR,
-		GNOMEUIINFO_MENU_QUIT_ITEM(gameQuit, this),
-		GNOMEUIINFO_END
-	};
-
-	gameMenuPtr = game_menu;
-	
-	static GnomeUIInfo settings_menu[] = 
-	{
-		GNOMEUIINFO_MENU_PREFERENCES_ITEM(gameProperties, this),
-		GNOMEUIINFO_END
-	};
-
-	gameSettingsPtr = settings_menu;
-	
-	GnomeUIInfo help_menu[] = 
-	{
-		GNOMEUIINFO_HELP((gpointer)"gnometris"),
-		GNOMEUIINFO_MENU_ABOUT_ITEM(gameAbout, this),
-		GNOMEUIINFO_END
-	};
-
-	GnomeUIInfo mainmenu[] = 
-	{
-		GNOMEUIINFO_MENU_GAME_TREE(game_menu),
-		GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu),
-		GNOMEUIINFO_MENU_HELP_TREE(help_menu),
-		GNOMEUIINFO_END
-	};
+	g_signal_connect (G_OBJECT (w), "focus_out_event",
+			  G_CALLBACK (focusOut), this);
+	g_signal_connect (G_OBJECT (w), "configure_event",
+			  G_CALLBACK (configure), this);
 
 	line_fill_height = 0;
 	line_fill_prob = 5;
 
 	/* init gconf */
 	gconf_client = gconf_client_get_default ();
-	games_gconf_sanity_check_string (gconf_client, KEY_BLOCK_PIXMAP);
 
 	gconf_client_add_dir (gconf_client, KEY_OPTIONS_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
 	gconf_client_notify_add (gconf_client, KEY_OPTIONS_DIR, gconfNotify, this, NULL, NULL);
@@ -194,24 +197,62 @@ Tetris::Tetris(int cmdlLevel):
 	gconf_client_add_dir (gconf_client, KEY_CONTROLS_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
 	gconf_client_notify_add (gconf_client, KEY_CONTROLS_DIR, gconfNotify, this, NULL, NULL);
 
+
+	width = gconf_client_get_int (gconf_client, KEY_WIDTH, NULL);
+	if (width <= 0)
+		width = 440;
+	height = gconf_client_get_int (gconf_client, KEY_HEIGHT, NULL);
+	if (height <= 0)
+		height = 550;
+
+	gtk_window_set_default_size (GTK_WINDOW (w), width, height);
+
+	preview = new Preview ();
+	field = new Field();
+
 	initOptions ();
 
-	gnome_app_create_menus(GNOME_APP(w), mainmenu);
+	/* prepare menus */
+	games_stock_init ();
+	action_group = gtk_action_group_new ("MenuActions");
+	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (action_group, actions, G_N_ELEMENTS (actions), this);
+	ui_manager = gtk_ui_manager_new ();
+	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+	gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
+	accel_group = gtk_ui_manager_get_accel_group (ui_manager);
+	gtk_window_add_accel_group (GTK_WINDOW (w), accel_group);
+
+	new_game_action = gtk_action_group_get_action (action_group, "NewGame");
+	pause_action = gtk_action_group_get_action (action_group, "Pause");
+	resume_action = gtk_action_group_get_action (action_group, "Resume");
+	scores_action = gtk_action_group_get_action (action_group, "Scores");
+	end_game_action = gtk_action_group_get_action (action_group, "EndGame");
+	preferences_action = gtk_action_group_get_action (action_group, "Preferences");
+
+	games_stock_set_pause_actions (pause_action, resume_action);
+
+	menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
 
 	GtkWidget * hb = gtk_hbox_new(FALSE, 0);
-	gnome_app_set_contents(GNOME_APP(w), hb);
 
-	field = new Field(/*ops*/);
-	ops = new BlockOps(field);
+	vbox = gtk_vbox_new (FALSE, 0);
+	gnome_app_set_contents (GNOME_APP (w), vbox);
+	gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hb, TRUE, TRUE, 0);
+
+	aspect_frame = gtk_aspect_frame_new (NULL, 0.5, 0.5, (float) COLUMNS / (float) LINES, FALSE);
+	gtk_frame_set_shadow_type (GTK_FRAME (aspect_frame), GTK_SHADOW_NONE);
+	gtk_container_add (GTK_CONTAINER (aspect_frame), field->getWidget());
 
 	gtk_widget_set_events(w, gtk_widget_get_events(w) | 
 						  GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
-	
+
 	GtkWidget *vb1 = gtk_vbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(vb1), 10);
-	gtk_box_pack_start_defaults(GTK_BOX(vb1), field->getWidget());
-	gtk_box_pack_start(GTK_BOX(hb), vb1, 0, 0, 0);
-	field->show();
+	gtk_box_pack_start_defaults(GTK_BOX(vb1), aspect_frame);
+	gtk_box_pack_start_defaults(GTK_BOX(hb), vb1);
+
 	setupPixmap();
 
 	g_signal_connect (w, "key_press_event", G_CALLBACK (keyPressHandler), this);
@@ -221,114 +262,51 @@ Tetris::Tetris(int cmdlLevel):
 	gtk_container_set_border_width(GTK_CONTAINER(vb2), 10);
 	gtk_box_pack_end(GTK_BOX(hb), vb2, 0, 0, 0);
 	
-	preview = new Preview();
-	
 	gtk_box_pack_start(GTK_BOX(vb2), preview->getWidget(), FALSE, FALSE, 0);
 	
 	scoreFrame = new ScoreFrame(cmdlineLevel);
 	
 	gtk_box_pack_end(GTK_BOX(vb2), scoreFrame->getWidget(), TRUE, FALSE, 0);
+	high_scores = new HighScores ();
 
 	setOptions ();
-        setupScoreState ();
+	setupScoreState ();
 
 	themeList = NULL;
 	
 	gtk_widget_show(hb);
 	gtk_widget_show(vb1);
 	gtk_widget_show(vb2);
+	gtk_widget_show(aspect_frame);
+	gtk_widget_show(field->getWidget());
 	scoreFrame->show();
 	gtk_widget_show(w);
 
-	gtk_widget_set_sensitive(gameMenuPtr[1].widget, FALSE);
-	gtk_widget_set_sensitive(gameMenuPtr[4].widget, FALSE);
-	gtk_widget_set_sensitive(gameSettingsPtr[0].widget, TRUE);
-
-        pauseMessage = gnome_canvas_item_new (gnome_canvas_root(GNOME_CANVAS(field->getWidget())),
-                                              gnome_canvas_text_get_type(),
-                                              "fill_color",
-                                              "white",
-                                              "x", COLUMNS*BLOCK_SIZE/2.0,
-                                              "y", LINES*BLOCK_SIZE/2.0,
-                                              "text", _("Paused"),
-                                              "size_points", 36.0,
-                                              NULL
-                                              );
-
-	/* Since gnome_canvas doesn't support setting the size of text in
-	 * pixels (read the source where the "size" parameter gets set)
-	 * and pango isn't forthcoming about how it scales things (see
-	 * http://mail.gnome.org/archives/gtk-i18n-list/2003-August/msg00001.html
-	 * and bug #119081). We guess at the size, see what size it is rendered
-	 * to and then adjust the point size to fit. 36.0 points is pretty
-	 * close for 96 dpi . */
-
-	gnome_canvas_item_get_bounds (pauseMessage, &x1, &y1, &x2, &y2);
-	width = x2 - x1;
-	/* 0.8 is the fraction of the screen we want to use and 36.0 is
-	 * the guess we use previously for the point size. */
-	pts = 0.8*36.0*COLUMNS*BLOCK_SIZE/width;
-	gnome_canvas_item_set (pauseMessage, "size_points", pts, 0);
-
-        gnome_canvas_item_hide (pauseMessage);
-
-        gameoverMessage = gnome_canvas_item_new (gnome_canvas_root(GNOME_CANVAS(field->getWidget())),
-                                                 gnome_canvas_text_get_type(),
-                                                 "fill_color",
-                                                 "white",
-                                                 "x", COLUMNS*BLOCK_SIZE/2.0,
-                                                 "y", LINES*BLOCK_SIZE/2.0,
-                                                 "text", _("Game Over"),
-                                                 "size_points", 36.0,
-                                                 NULL
-                                                 );
-
-	gnome_canvas_item_get_bounds (gameoverMessage, &x1, &y1, &x2, &y2);
-	width = x2 - x1;
-	/* 0.9 is the fraction of the screen we want to use and 36.0 is
-	 * the guess we use previously for the point size. */
-	pts = 0.9*36.0*COLUMNS*BLOCK_SIZE/width;
-	gnome_canvas_item_set (gameoverMessage, "size_points", pts, 0);
-
-        gnome_canvas_item_hide (gameoverMessage);
+	gtk_action_set_sensitive(pause_action, FALSE);
+	gtk_action_set_sensitive(end_game_action, FALSE);
+	gtk_action_set_sensitive(preferences_action, TRUE);
 }
 
 Tetris::~Tetris()
 {
-	delete ops;
 	delete field;
 	delete preview;
 	delete scoreFrame;
 
-	if (image)
-		g_object_unref (G_OBJECT (image));
 	if (bgimage)
 		g_object_unref (G_OBJECT (bgimage));		
 
-	if (blockPixmap)
-		g_free(blockPixmap);
 	if (bgPixmap)
 		g_free(bgPixmap);
-
-	delete[] pic;
 }
 
 void
 Tetris::setupScoreState ()
 {
-        gchar **names = NULL;
-        gfloat *scores = NULL;
-        time_t *scoretimes = NULL;
-	gint top = 0;
-
-	top = gnome_score_get_notable ("gnometris", NULL, &names, &scores, &scoretimes);
-	if (top > 0) {
-		gtk_widget_set_sensitive (gameMenuPtr[3].widget, TRUE);
-		g_strfreev (names);
-		g_free (scores);
-		g_free (scoretimes);
+	if (high_scores->empty ()) {
+		gtk_action_set_sensitive (scores_action, FALSE);
 	} else {
-		gtk_widget_set_sensitive (gameMenuPtr[3].widget, FALSE);
+		gtk_action_set_sensitive (scores_action, TRUE);
 	}
 }
 
@@ -336,10 +314,12 @@ void
 Tetris::setupdialogDestroy(GtkWidget *widget, void *d)
 {
 	Tetris *t = (Tetris*) d;
-	if (t->setupdialog)
+	if (t->setupdialog) {
+		delete t->theme_preview;
 		gtk_widget_destroy(t->setupdialog);
+	}
 	t->setupdialog = 0;
-	gtk_widget_set_sensitive(t->gameMenuPtr[0].widget, TRUE);
+	gtk_action_set_sensitive(t->new_game_action, TRUE);
 }
 
 void
@@ -351,60 +331,6 @@ Tetris::setupdialogResponse (GtkWidget *dialog, gint response_id, void *d)
 void
 Tetris::setupPixmap()
 {
-	gchar *pixname, *fullpixname;
-	
-	pixname = g_build_filename ("gnometris", blockPixmap, NULL);
-	fullpixname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, pixname, FALSE, NULL);
-	g_free (pixname);
-
-	if (!g_file_test (fullpixname, G_FILE_TEST_EXISTS))
-	{
-		g_free (fullpixname);
-		fullpixname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, "gnometris/7blocks-gw.png", FALSE, NULL);
-	}
-	
-	if (!g_file_test (fullpixname, G_FILE_TEST_EXISTS))
-	{
-		GtkWidget *widget = gtk_message_dialog_new (NULL,
-						       GTK_DIALOG_DESTROY_WITH_PARENT,
-						       GTK_MESSAGE_ERROR,
-						       GTK_BUTTONS_OK,
-						       _("Could not find the theme: \n%s\n\nPlease check your gnome-games installation"), fullpixname);
-		gtk_dialog_run (GTK_DIALOG (widget));
-		exit(1);
-	}
-
-	if(image)
-		g_object_unref (G_OBJECT (image));
-
-	image = gdk_pixbuf_new_from_file(fullpixname, NULL);
-
-	if (image == NULL) {
-		GtkWidget *widget= gtk_message_dialog_new (NULL,
-						       GTK_DIALOG_DESTROY_WITH_PARENT,
-						       GTK_MESSAGE_ERROR,
-						       GTK_BUTTONS_OK,
-						       _("Can't load the image: \n%s\n\nPlease check your gnome-games installation"), fullpixname);
-		gtk_dialog_run (GTK_DIALOG (widget));
-		exit (1);
-	}
-	g_free (fullpixname);
-
-	BLOCK_SIZE = gdk_pixbuf_get_height(image);
-	nr_of_colors = gdk_pixbuf_get_width(image) / BLOCK_SIZE;
-
-	for (int i = 0; i < tableSize; ++i)
-	{
-		if (pic[i])
-			g_object_unref (G_OBJECT (pic[i]));
-
-		pic[i] = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, BLOCK_SIZE, BLOCK_SIZE);
-
-		gdk_pixbuf_copy_area (image, (i % nr_of_colors) * BLOCK_SIZE, 0,
-					BLOCK_SIZE, BLOCK_SIZE, pic[i], 0, 0);
-
-	}
-
 	if (bgimage)
 		g_object_unref (G_OBJECT (bgimage));
 
@@ -462,14 +388,16 @@ Tetris::setupPixmap()
 
 	if (field)
 	{
-		field->updateSize (bgimage, &bgcolour);
-		gtk_widget_queue_resize (field->getWidget());
+		if (bgimage)
+			field->setBackground (bgimage);
+		else
+			field->setBackground (&bgcolour);
 	}
 	
 	if (preview)
 	{
-		preview->updateSize ();
-		gtk_widget_queue_resize (field->getWidget());
+		/* FIXME: We should do an update once the preview actually
+		 * uses the background pixbuf. */
 	}
 }
 
@@ -538,10 +466,9 @@ Tetris::initOptions ()
 {
 	gchar *bgcolourstr;
 
-	if (blockPixmap)
-		g_free (blockPixmap);
-
-	blockPixmap = gconfGetString (gconf_client, KEY_BLOCK_PIXMAP, "7blocks-tig.png");
+	themeno = themeNameToNumber (gconfGetString (gconf_client, KEY_THEME, "plain"));
+	field->setTheme (themeno);
+	preview->setTheme (themeno);
 
 	startingLevel = gconfGetInt (gconf_client, KEY_STARTING_LEVEL, 1);
 	if (startingLevel < 1) 
@@ -555,6 +482,10 @@ Tetris::initOptions ()
 		sound->turnOff ();
 
 	do_preview = gconfGetBoolean (gconf_client, KEY_DO_PREVIEW, TRUE);
+	
+	if (preview) {
+		preview->enable(do_preview);
+	}
 
 	random_block_colors = gconfGetBoolean (gconf_client, KEY_RANDOM_BLOCK_COLORS, TRUE);
 
@@ -597,6 +528,11 @@ Tetris::setOptions ()
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (do_preview_toggle), do_preview);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (random_block_colors_toggle), random_block_colors);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rotate_counter_clock_wise_toggle), rotateCounterClockWise);
+
+		if (theme_preview) {
+			theme_preview->setTheme (themeno);
+			gtk_widget_queue_draw(theme_preview->getWidget());
+		}
 	}
 
 	scoreFrame->setLevel (startingLevel);
@@ -642,13 +578,12 @@ Tetris::setSelection(GtkWidget *widget, void *data)
 	Tetris *t;
 	GList * item;
 
-      	t = (Tetris *)g_object_get_data (G_OBJECT (widget), TETRIS_OBJECT);
+      	t = (Tetris *)data;
 
-	item = g_list_nth (t->themeList,
-			   gtk_combo_box_get_active (GTK_COMBO_BOX (widget)));
-
-	gconf_client_set_string (t->gconf_client, KEY_BLOCK_PIXMAP,
-				 (char *)item->data, NULL);
+	t->themeno = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
+	t->field->setTheme (t->themeno);
+	gconf_client_set_string (t->gconf_client, KEY_THEME,
+				 ThemeTable[t->themeno].id, NULL);
 }
 
 void
@@ -707,7 +642,9 @@ Tetris::fillMenu(GtkWidget *menu, char *pixname, char *dirname,
 	{
 		s = g_strdup(e->d_name);
 
-		if (!(strstr (e->d_name, ".png") || strstr (e->d_name, ".jpg"))) 
+		if (!(strstr (e->d_name, ".png") || 
+		      strstr (e->d_name, ".jpg") ||
+		      strstr (e->d_name, ".svg"))) 
 		{
 			free(s);
 			continue;
@@ -734,7 +671,7 @@ Tetris::fillMenu(GtkWidget *menu, char *pixname, char *dirname,
 }
 
 int 
-Tetris::gameProperties(GtkWidget *widget, void *d)
+Tetris::gameProperties(GtkAction *action, void *d)
 {
 	GtkWidget *notebook;
 	GtkWidget *vbox;
@@ -744,7 +681,7 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	GtkWidget *fvbox;
 	GtkObject *adj;
 	GtkWidget *controls_list;
-        
+
 	Tetris *t = (Tetris*) d;
 	
 	if (t->setupdialog) {
@@ -754,37 +691,42 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 
 	/* create the dialog */
 	t->setupdialog =
-		gtk_dialog_new_with_buttons(_("Gnometris preferences"), 
+		gtk_dialog_new_with_buttons(_("Gnometris Preferences"), 
 					    GTK_WINDOW (t->w),
 					    (GtkDialogFlags)0,
 					    GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 					    NULL);
 	gtk_dialog_set_has_separator (GTK_DIALOG (t->setupdialog), FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (t->setupdialog), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (t->setupdialog), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (t->setupdialog)->vbox), 2);
 	g_signal_connect (t->setupdialog, "close",
 			  G_CALLBACK (setupdialogDestroy), d);
 	g_signal_connect (t->setupdialog, "response",
 			  G_CALLBACK (setupdialogResponse), d);
 
 	notebook = gtk_notebook_new ();
+	gtk_container_set_border_width (GTK_CONTAINER (notebook), 5);
 	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(t->setupdialog)->vbox), notebook,
 			    TRUE, TRUE, 0);
 
 	/* game page */
-	vbox = gtk_vbox_new (FALSE, 0);
-	label = gtk_label_new_with_mnemonic (_("_Game"));
+	vbox = gtk_vbox_new (FALSE, 18);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
+	label = gtk_label_new (_("Game"));
 	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, label);
 
 	frame = games_frame_new (_("Setup"));
 	table = gtk_table_new (3, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), 0);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
 
 	/* pre-filled rows */
-	label = gtk_label_new(_("Number of pre-filled rows:"));
+	label = gtk_label_new_with_mnemonic (_("_Number of pre-filled rows:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
+	                  (GtkAttachOptions) GTK_FILL, 
+			  (GtkAttachOptions) 0,
+			  0, 0);
 
 	adj = gtk_adjustment_new (t->line_fill_height, 0, LINES-1, 1, 5, 0);
 	t->fill_height_spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 10, 0);
@@ -795,26 +737,34 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	g_signal_connect (t->fill_height_spinner, "value_changed",
 			  G_CALLBACK (lineFillHeightChanged), t);
 	gtk_table_attach_defaults (GTK_TABLE (table), t->fill_height_spinner, 1, 2, 0, 1);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), t->fill_height_spinner);
 
 	/* pre-filled rows density */
-	label = gtk_label_new (_("Density of blocks in a pre-filled row:"));
+	label = gtk_label_new_with_mnemonic (_("_Density of blocks in a pre-filled row:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
+	                  (GtkAttachOptions) GTK_FILL, 
+			  (GtkAttachOptions) 0,
+			  0, 0);
 
 	adj = gtk_adjustment_new (t->line_fill_prob, 0, 10, 1, 5, 0);
 	t->fill_prob_spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 10, 0);
 	gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (t->fill_prob_spinner),
-					  GTK_UPDATE_ALWAYS);
+					  GTK_UPDATE_ALWAYS);			  
 	gtk_spin_button_set_snap_to_ticks
 		(GTK_SPIN_BUTTON (t->fill_prob_spinner), TRUE);
 	g_signal_connect (t->fill_prob_spinner, "value_changed",
 		          G_CALLBACK (lineFillProbChanged), t);
 	gtk_table_attach_defaults (GTK_TABLE (table), t->fill_prob_spinner, 1, 2, 1, 2);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), t->fill_prob_spinner);
 
 	/* starting level */
-	label = gtk_label_new (_("Starting Level:"));
+	label = gtk_label_new_with_mnemonic (_("_Starting level:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
+	                  (GtkAttachOptions) GTK_FILL, 
+			  (GtkAttachOptions) 0,
+			  0, 0);
 
 	adj = gtk_adjustment_new (t->startingLevel, 1, 10, 1, 5, 10);
 	t->sentry = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 10, 0);
@@ -824,6 +774,7 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	g_signal_connect (t->sentry, "value_changed",
 			  G_CALLBACK (startingLevelChanged), t);
 	gtk_table_attach_defaults (GTK_TABLE (table), t->sentry, 1, 2, 2, 3);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), t->sentry);
 
 	gtk_container_add (GTK_CONTAINER (frame), table);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, 
@@ -834,17 +785,16 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 
 	/* sound */
 	t->sound_toggle =
-		gtk_check_button_new_with_label (_("Enable sounds"));
+		gtk_check_button_new_with_mnemonic (_("_Enable sounds"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (t->sound_toggle),
 				     sound->isOn ());
 	g_signal_connect (t->sound_toggle, "clicked",
 			  G_CALLBACK (setSound), d);
 	gtk_box_pack_start (GTK_BOX (fvbox), t->sound_toggle, 0, 0, 0);
 	
-
 	/* preview next block */
 	t->do_preview_toggle =
-		gtk_check_button_new_with_label (_("Preview next block"));
+		gtk_check_button_new_with_mnemonic (_("_Preview next block"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (t->do_preview_toggle),
 				     do_preview);
 	g_signal_connect (t->do_preview_toggle, "clicked",
@@ -853,7 +803,7 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 
 	/* random blocks */
 	t->random_block_colors_toggle =
-		gtk_check_button_new_with_label(_("Use random block colors"));
+		gtk_check_button_new_with_mnemonic (_("_Use random block colors"));
 	gtk_toggle_button_set_active
 		(GTK_TOGGLE_BUTTON (t->random_block_colors_toggle),
 		 random_block_colors);
@@ -864,7 +814,7 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 
 	/* rotate counter clock wise */
  	t->rotate_counter_clock_wise_toggle =
-		gtk_check_button_new_with_label (_("Rotate blocks counterclockwise"));
+		gtk_check_button_new_with_mnemonic (_("_Rotate blocks counterclockwise"));
  	gtk_toggle_button_set_active
 		(GTK_TOGGLE_BUTTON (t->rotate_counter_clock_wise_toggle),
 		 rotateCounterClockWise);
@@ -881,29 +831,17 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	table = gtk_table_new (2, 2, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (table), 0);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
-
-	/* Block pixmap */
-	label = gtk_label_new(_("Block image:"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
-
-	GtkWidget *omenu = gtk_combo_box_new_text ();
-	g_object_set_data (G_OBJECT (omenu), TETRIS_OBJECT, t);	
-	t->fillMenu (omenu, t->blockPixmap, "gnometris", &(t->themeList));
-	g_signal_connect (omenu, "changed", G_CALLBACK (setSelection), NULL);
-	gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
-
-	gtk_container_add (GTK_CONTAINER (frame), table);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, 
-			    FALSE, FALSE, 0);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
 
 	/* controls page */
 	vbox = gtk_vbox_new (FALSE, 0);
-	label = gtk_label_new_with_mnemonic (_("_Controls"));
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
+	label = gtk_label_new (_("Controls"));
 	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, label);
 
-	frame = games_frame_new (_("Keyboard controls"));
+	frame = games_frame_new (_("Keyboard Controls"));
+	gtk_container_add (GTK_CONTAINER (vbox), frame);
+
 	fvbox = gtk_vbox_new (FALSE, 6);
 	gtk_container_add (GTK_CONTAINER (frame), fvbox);
 
@@ -919,15 +857,50 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 
 	gtk_box_pack_start (GTK_BOX (fvbox), controls_list, TRUE, TRUE, 0);
 
+	/* theme page */
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
+	label = gtk_label_new (_("Theme"));
+	gtk_notebook_append_page (GTK_NOTEBOOK(notebook), vbox, label);
+
+	frame = games_frame_new (_("Block Style"));
 	gtk_container_add (GTK_CONTAINER (vbox), frame);
-	
+
+	fvbox = gtk_vbox_new (FALSE, 6);
+	gtk_container_add (GTK_CONTAINER (frame), fvbox);
+
+	GtkWidget *omenu = gtk_combo_box_new_text ();
+	const ThemeTableEntry *entry = ThemeTable;
+	while (entry->id) {
+		gtk_combo_box_append_text (GTK_COMBO_BOX (omenu), entry->name);
+		entry++;
+	}
+	gtk_combo_box_set_active (GTK_COMBO_BOX (omenu), t->themeno);
+	g_signal_connect (omenu, "changed", G_CALLBACK (setSelection), t);
+	gtk_box_pack_start (GTK_BOX (fvbox), omenu, FALSE, FALSE, 0);
+
+	t->theme_preview = new Preview();
+	t->theme_preview->setTheme (t->themeno);
+	gtk_box_pack_start(GTK_BOX(fvbox), t->theme_preview->getWidget(), TRUE, TRUE, 0);
+
+	t->theme_preview->previewBlock(4, 0, 0);
+
 	gtk_widget_show_all (t->setupdialog);
-	gtk_widget_set_sensitive(t->gameMenuPtr[0].widget, FALSE);
+	gtk_action_set_sensitive(t->new_game_action, FALSE);
+
+	return TRUE;
+}
+
+int 
+Tetris::focusOut(GtkWidget *widget, GdkEvent *e, Tetris *t)
+{
+	if (t->inPlay && !t->paused)
+		t->togglePause();
 	return TRUE;
 }
 
 int
-Tetris::gamePause(GtkWidget *widget, void *d)
+Tetris::gamePause(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
 	t->togglePause();
@@ -935,7 +908,7 @@ Tetris::gamePause(GtkWidget *widget, void *d)
 }
 
 int
-Tetris::gameEnd(GtkWidget *widget, void *d)
+Tetris::gameEnd(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
 
@@ -947,13 +920,13 @@ Tetris::gameEnd(GtkWidget *widget, void *d)
 }
 
 int
-Tetris::gameQuit(GtkWidget *widget, void *d)
+Tetris::gameQuit(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
 
 	/* Record the score if the game isn't over. */
 	if (t->inPlay && (t->scoreFrame->getScore() > 0))
-		gnome_score_log(t->scoreFrame->getScore(), 0, TRUE);
+		t->high_scores->add (t->scoreFrame->getScore());
 
 	if (t->w)
 		gtk_widget_destroy(t->w);
@@ -977,11 +950,14 @@ Tetris::generateTimer(int level)
 void
 Tetris::manageFallen()
 {
-	ops->fallingToLaying();
+	field->fallingToLaying();
 	sound->playSound (SOUND_LAND);
 
 	int levelBefore = scoreFrame->getLevel();
-	ops->checkFullLines(scoreFrame);
+
+	int fullLines = field->checkFullLines();
+	if (fullLines > 0)
+		scoreFrame->incLines(fullLines);
 	int levelAfter = scoreFrame->getLevel();
 	if (levelAfter != levelBefore) 
 		sound->playSound (SOUND_GNOMETRIS);
@@ -1002,12 +978,13 @@ Tetris::timeoutHandler(void *d)
  	if (t->onePause)
  	{
 		t->onePause = false;
-		gtk_widget_queue_draw(t->field->getWidget());
+		t->field->redraw();
 	}
  	else
 	{
-		bool res = t->ops->moveBlockDown();
-		gtk_widget_queue_draw(t->field->getWidget());
+		bool res = t->field->moveBlockDown();
+
+		t->field->redraw();
 
 		if (res)
 		{
@@ -1046,17 +1023,17 @@ Tetris::keyPressHandler(GtkWidget *widget, GdkEvent *event, Tetris *t)
 		return FALSE;
 
 	if (keyval == t->moveLeft) {
-		res = t->ops->moveBlockLeft();
+		res = t->field->moveBlockLeft();
 		if (res)
 			sound->playSound (SOUND_SLIDE);
 		t->onePause = false;
 	} else if (keyval == t->moveRight) {
-		res = t->ops->moveBlockRight();
+		res = t->field->moveBlockRight();
 		if (res)
 			sound->playSound (SOUND_SLIDE);
 		t->onePause = false;
 	} else if (keyval == t->moveRotate) {
-		res = t->ops->rotateBlock(rotateCounterClockWise);
+		res = t->field->rotateBlock(rotateCounterClockWise);
 		if (res)
 			sound->playSound (SOUND_TURN);
 		t->onePause = false;
@@ -1071,13 +1048,15 @@ Tetris::keyPressHandler(GtkWidget *widget, GdkEvent *event, Tetris *t)
 	} else if (keyval == t->moveDrop) {
 		if (!t->dropBlock) {
 			t->dropBlock = true;
-			bonus = t->ops->dropBlock();
+			bonus = t->field->dropBlock();
 			t->scoreFrame->incScore(bonus);
 			t->manageFallen();
 			res = TRUE;
 		}        
 	}
 
+	t->field->redraw();
+
 	return res;
 }
 
@@ -1301,26 +1280,31 @@ Tetris::dragDrop(GtkWidget *widget, GdkDragContext *context,
 	gnome_vfs_close (inhandle);
  error_exit:
 	return;
-
 }
 
 void
 Tetris::togglePause()
 {
 	paused = !paused;
-        if (paused) {
-                gnome_canvas_item_show (pauseMessage);
-                gnome_canvas_item_raise_to_top (pauseMessage);
-        } else
-                gnome_canvas_item_hide (pauseMessage);
+
+	if (paused)
+		field->showPauseMessage();
+	else
+		field->hidePauseMessage();
+
+	gtk_action_set_sensitive (pause_action, !paused);
+	gtk_action_set_sensitive (resume_action, paused);
+	gtk_action_set_visible (pause_action, !paused);
+	gtk_action_set_visible (resume_action, paused);
 }
 
 void
 Tetris::generate()
 {
-	if (ops->generateFallingBlock())
+	if (field->generateFallingBlock())
 	{
-		ops->putBlockInField(false);
+		field->putBlockInField(false);
+		preview->previewBlock(blocknr_next, rot_next, color_next);
 		gtk_widget_queue_draw(preview->getWidget());
 		onePause = true;
 	}
@@ -1337,39 +1321,30 @@ Tetris::generate()
 void
 Tetris::endOfGame()
 {
-	gtk_widget_set_sensitive(gameMenuPtr[1].widget, FALSE);
-	gtk_widget_set_sensitive(gameMenuPtr[4].widget, FALSE);
-	gtk_widget_set_sensitive(gameSettingsPtr[0].widget, TRUE);
+	if (paused) togglePause();
+	gtk_action_set_sensitive (pause_action, FALSE);
+	gtk_action_set_sensitive (end_game_action, FALSE);
+	gtk_action_set_sensitive (preferences_action, TRUE);
+
 	color_next = -1;
 	blocknr_next = -1;
 	rot_next = -1;
-        gnome_canvas_item_hide (pauseMessage);
-        gnome_canvas_item_show (gameoverMessage);
-        gnome_canvas_item_raise_to_top (gameoverMessage);
+	preview->previewBlock(-1, -1, -1);
+	gtk_widget_queue_draw(preview->getWidget());
+	field->hidePauseMessage();
+	field->showGameOverMessage();
 	sound->playSound (SOUND_GAMEOVER);
 	inPlay = false;
 
 	if (scoreFrame->getScore() > 0) 
 	{
-		int pos = gnome_score_log(scoreFrame->getScore(), 0, TRUE);
-		showScores("Gnometris", pos);
-	}
-}
-
-void
-Tetris::showScores(gchar *title, guint pos)
-{
-	GtkWidget *dialog;
-
-	dialog = gnome_scores_display(title, "gnometris", 0, pos);
-	if (dialog != NULL) {
-		gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW(this->getWidget()));
-		gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
+		int pos = high_scores->add (scoreFrame->getScore());
+		high_scores->show (pos);
 	}
 }
 
 int
-Tetris::gameNew(GtkWidget *widget, void *d)
+Tetris::gameNew(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
 
@@ -1381,7 +1356,7 @@ Tetris::gameNew(GtkWidget *widget, void *d)
 		/* Catch the case where we started a new game without
 		 * finishing the old one. */
 		if ((t->scoreFrame->getScore() > 0) && t->inPlay)
-			gnome_score_log(t->scoreFrame->getScore(), 0, TRUE);
+			t->high_scores->add (t->scoreFrame->getScore());
 	}
 
 	t->inPlay = true;
@@ -1394,86 +1369,75 @@ Tetris::gameNew(GtkWidget *widget, void *d)
 	t->scoreFrame->setStartingLevel(level);
 
 	t->timeoutId = g_timeout_add_full(0, 1000 - 100 * (level - 1), timeoutHandler, t, NULL);
-	t->ops->emptyField(t->line_fill_height,t->line_fill_prob);
+	t->field->emptyField(t->line_fill_height,t->line_fill_prob);
 
 	t->scoreFrame->resetLines();
 	t->paused = false;
 	
-	t->ops->generateFallingBlock();
-	gtk_widget_queue_draw(t->field->getWidget());
+	t->field->generateFallingBlock();
+	//gtk_widget_queue_draw(t->field->getWidget());
+	t->field->redraw();
+	t->preview->previewBlock(blocknr_next, rot_next, color_next);
 	gtk_widget_queue_draw(t->preview->getWidget());
 
-	gtk_widget_set_sensitive(t->gameMenuPtr[1].widget, TRUE);
-	gtk_widget_set_sensitive(t->gameMenuPtr[4].widget, TRUE);
-	gtk_widget_set_sensitive(t->gameSettingsPtr[0].widget, FALSE);
+	gtk_action_set_sensitive(t->pause_action, TRUE);
+	gtk_action_set_sensitive(t->end_game_action, TRUE);
+	gtk_action_set_sensitive(t->preferences_action, FALSE);
+
+	t->field->hidePauseMessage();
+	t->field->hideGameOverMessage();
 
-        gnome_canvas_item_hide (t->pauseMessage);
-        gnome_canvas_item_hide (t->gameoverMessage);
-        
 	sound->playSound (SOUND_GNOMETRIS);
 
 	return TRUE;
 }
 
 int
-Tetris::gameAbout(GtkWidget *widget, void *d)
+Tetris::gameHelp(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
-	GdkPixbuf *pixbuf = NULL;
-	static GtkWidget *about = NULL;
 
-	const gchar *authors[] = {"J. Marcin Gorycki", 0};
-
-	const gchar *documenters[] = {
-		NULL
-	};
-
-	const gchar *translator_credits = _("translator-credits");
-
-	if (about != NULL) {
-		gtk_window_present (GTK_WINDOW(about));
-		return TRUE;
-	}
-        {
-		char *filename = NULL;
-
-		filename = gnome_program_locate_file (NULL,
-				GNOME_FILE_DOMAIN_APP_PIXMAP, 
-				"gnome-gtetris.png",
-				TRUE, NULL);
-		if (filename != NULL)
-		{
-			pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
-			g_free (filename);
-		}
-	}
-
-	about = gnome_about_new("Gnometris", 
-				VERSION, 
-				"(C) 2000 J. Marcin Gorycki",
-				_("Written for my wife, Matylda\n"
-				  "Send comments and bug reports to: "
-				  "janusz.gorycki@intel.com"),
-				(const char **)authors,
-				(const char **)documenters,
-				strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
-				pixbuf);
-	
-	if (pixbuf != NULL)
-		gdk_pixbuf_unref (pixbuf);
-	
-	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (t->getWidget()));
-	g_signal_connect (G_OBJECT (about), "destroy", G_CALLBACK (gtk_widget_destroyed), &about);
-	gtk_widget_show(about);
+	gnome_help_display ("gnometris.xml", NULL, NULL);
 
 	return TRUE;
 }
 
 int
-Tetris::gameTopTen(GtkWidget *widget, void *d)
+Tetris::gameAbout(GtkAction *action, void *d)
 {
 	Tetris *t = (Tetris*) d;
-	t->showScores("Gnometris", 0);
+
+	const gchar * const authors[] = { "J. Marcin Gorycki", NULL };
+
+	gtk_show_about_dialog (GTK_WINDOW (t->getWidget()),
+			       "name", _("Gnometris"),
+			       "version", VERSION,
+			       "comments", _("Written for my wife, Matylda\n"
+					     "Send comments and bug reports to: \n"
+					     "janusz.gorycki@intel.com"),
+				"copyright", "Copyright \xc2\xa9 2000 J. Marcin Gorycki",
+				"authors", authors,				
+				"translator_credits", _("translator-credits"),
+				NULL);
 
 	return TRUE;
 }
+
+int
+Tetris::gameTopTen(GtkAction *action, void *d)
+{
+	Tetris *t = (Tetris*) d;
+	t->high_scores->show(0);
+
+	return TRUE;
+}
+
+gboolean Tetris::configure (GtkWidget *widget, GdkEventConfigure *event, Tetris *t)
+{
+
+	gconf_client_set_int (t->gconf_client, KEY_WIDTH, event->width, NULL);
+	gconf_client_set_int (t->gconf_client, KEY_HEIGHT, event->height, 
+			      NULL);
+
+	return FALSE;
+}
