@@ -10,9 +10,6 @@
  *
  *   Gnome specific yahtzee routines.
  *
- *   Curses routines are located in cyahtzee.c (cyahtzee.c is 
- *   very broken at the moment 980617).
- *
  *   Other gnome specific code is in setup.c and clist.c
  *
  *   Window manager independent routines are in yahtzee.c and computer.c
@@ -45,21 +42,20 @@
 #include <gconf/gconf-client.h>
 #include <libgnomeui/gnome-window-icon.h>
 
+#include <games-stock.h>
+
 #include "yahtzee.h"
 #include "gyahtzee.h"
 
-/* Define a sensible alternative to ngettext if we don't have it. Note that
- * this is only sensible in the context of gtali. */
-#ifndef HAVE_NGETTEXT
-#define ngettext(one,lots,n) gettext(lots)
-#endif
+#define DELAY_MS 600
 
 int GyahtzeeAbort = 0;  /* Abort program without playing game */
 
 static char *appID="gtali";
-static char *appName=N_("GNOME Tali");
+static char *appName=N_("Tali");
 static guint lastHighScore = 0;
-GtkWidget *ScoreList; 
+static guint last_timeout = 0;
+static gboolean ready_to_advance_player;
 
 #ifdef GNOMEPIXMAPDIR
 #define PP GNOMEPIXMAPDIR "/gtali/" 
@@ -72,39 +68,63 @@ static char *dicefiles[NUMBER_OF_PIXMAPS] = { PP "gnome-dice-1.svg",
                                               PP "gnome-dice-3.svg",
                                               PP "gnome-dice-4.svg",
                                               PP "gnome-dice-5.svg",
-                                              PP "gnome-dice-6.svg",
-                                              PP "gnome-dice-none.svg" };
+                                              PP "gnome-dice-6.svg" };
 
 static GtkWidget *dicePixmaps[NUMBER_OF_DICE][NUMBER_OF_PIXMAPS];
 
 GtkWidget *window;
-static GtkWidget *appbar, *diceBox[NUMBER_OF_DICE];
-static GtkWidget *diceTable;
+GtkWidget *ScoreList;
+static GtkWidget *statusbar;
+static GtkToolItem *diceBox[NUMBER_OF_DICE];
 static GtkWidget *rollLabel;
-
-static gint gnome_modify_dice (GtkWidget *widget, GdkEventButton *event,
+static GtkWidget *mbutton;
+static GtkAction *scores_action;
+static gint gnome_modify_dice (GtkWidget *widget,
                                gpointer data);
 static gint gnome_roll_dice (GtkWidget *widget, GdkEvent *event,
                              gpointer data);
 void update_score_state (void);
-static void undo_set_sensitive (gboolean state);
 static void UpdateRollLabel (void);
 
-void
-YahtzeeIdle (void)
+static void
+update_roll_button_sensitivity (void)
 {
-        while (gtk_events_pending ())
-                gtk_main_iteration ();
-}
+        gboolean state = FALSE;
+        gint i;
 
+        for (i=0; i < NUMBER_OF_DICE; i++)
+                state |= gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (diceBox[i]));
+
+        state &= NumberOfRolls < 3;
+        state &= !players[CurrentPlayer].comp;
+        
+        gtk_widget_set_sensitive (GTK_WIDGET (mbutton), state);
+}
 
 static void
 CheerWinner (void)
 {
         int winner;
+        int i;
+
+	ShowoffPlayer (ScoreList,CurrentPlayer,0);
 
         winner = FindWinner ();
-        if ( (winner<NumberOfHumans) && !IsCheater ) {
+
+        if (winner < 0) {
+                for (i = 0; i < NumberOfPlayers; i++) {
+                        if (total_score (i) == -winner) {
+                                ShowoffPlayer (ScoreList, i, 1);
+                        }
+                }
+                
+                say (_("The game is a draw!"));
+                return;
+        }
+
+	ShowoffPlayer (ScoreList,winner,1);
+
+        if (winner<NumberOfHumans) {
                 lastHighScore = gnome_score_log ((guint)WinningScore, 
                                                  NULL, TRUE);
 		update_score_state ();
@@ -112,9 +132,6 @@ CheerWinner (void)
                         ShowHighScores ();
         }
 
-	ShowoffPlayer (ScoreList,CurrentPlayer,0);
-	ShowoffPlayer (ScoreList,winner,1);
-
         if (players[winner].name)
                 say (ngettext("%s wins the game with %d point",
                               "%s wins the game with %d points", WinningScore),
@@ -123,21 +140,55 @@ CheerWinner (void)
 	else
                 say (_("Game over!"));
 
-        undo_set_sensitive (FALSE);
+}
+
+static gboolean
+do_computer_turns (void)
+{
+        if (!players[CurrentPlayer].comp) {
+                last_timeout = 0;
+                return FALSE;
+        }
+
+        if (ready_to_advance_player){
+                NextPlayer ();
+                return TRUE;
+        }
+
+        if (players[CurrentPlayer].finished){
+                NextPlayer();
+                return TRUE;
+        }
+
+        ComputerRolling(CurrentPlayer);
+        if ( NoDiceSelected() || (NumberOfRolls>=NUM_ROLLS) ) {
+                ComputerScoring(CurrentPlayer);
+                ready_to_advance_player = TRUE;
+        } else {
+                RollSelectedDice();
+                UpdateRollLabel();
+        }
+
+        if (!DoDelay)
+                do_computer_turns();
+
+        return TRUE;
 }
 
 void 
 NextPlayer (void)
 {
         if (GameIsOver ()) {
+                if (last_timeout){
+                        g_source_remove (last_timeout);
+                        last_timeout = 0;
+                }
                 CheerWinner ();
                 return;
         }
 
-        undo_set_sensitive (TRUE);
-        
         NumberOfRolls = 0;
-
+        ready_to_advance_player = FALSE;
 	ShowoffPlayer (ScoreList,CurrentPlayer,0);
 
         /* Find the next player with rolls left */
@@ -159,11 +210,16 @@ NextPlayer (void)
 
         SelectAllDice ();
         RollSelectedDice ();
-
+	
         if (players[CurrentPlayer].comp) {
-                ComputerTurn (CurrentPlayer);
-                NextPlayer ();
-                return;
+                if (DoDelay){
+                        if (!last_timeout)
+                                last_timeout = g_timeout_add (DELAY_MS,
+                                                              (GSourceFunc)do_computer_turns,
+                                                              NULL);
+                } else {
+                       do_computer_turns();
+                }
         }
         UpdateRollLabel ();
 }
@@ -211,7 +267,7 @@ ShowPlayer (int num, int field)
 }
 
 static gint 
-quit_game (GtkWidget *widget, gpointer data)
+quit_game (GObject *object, gpointer data)
 {
         gtk_main_quit ();
 	return TRUE;
@@ -229,9 +285,8 @@ key_press (GtkWidget *widget, GdkEventKey * event, gpointer data)
         if ((offset < 0) || (offset >= NUMBER_OF_DICE)) {
                 return FALSE;
         }
-
-        /* I hate faking signal calls. */
-        gnome_modify_dice (NULL, NULL, &DiceValues[offset]);
+        
+	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON(diceBox[offset]), !DiceValues[offset].sel);
 
         return FALSE;
 }
@@ -241,8 +296,7 @@ GyahtzeeNewGame (void)
 {
         int i;
 
-        say (_("Select dice to re-roll, press Roll!,"
-               " or select score slot."));
+        say (_("Select dice to roll or choose a score slot."));
 
         NewGame();
         setup_score_list (ScoreList);
@@ -251,19 +305,11 @@ GyahtzeeNewGame (void)
 	for (i=0; i<NumberOfPlayers; i++)
                 ShowoffPlayer (ScoreList,i,0);
 	ShowoffPlayer (ScoreList,0,1);
-
-        /* All players are computers, start game immediately */
-        if (players[CurrentPlayer].comp) {
-                CurrentPlayer = NumberOfComputers - 1;
-                NextPlayer ();
-        }
-
-        undo_set_sensitive (FALSE);
 }
 
 
 static gint 
-new_game_callback (GtkWidget *widget, gpointer data)
+new_game_callback (GtkAction *action, gpointer data)
 {
         GyahtzeeNewGame ();
 	return FALSE;
@@ -279,87 +325,55 @@ UpdateRollLabel (void)
         
         g_string_printf(str, "<b>%s %d/3</b>", _("Roll"), NumberOfRolls);
         gtk_label_set_label(GTK_LABEL(rollLabel), str->str);
+	
+        update_roll_button_sensitivity();
 }
 
-static void
-UpdateDiePixmap (DiceInfo *d)
-{
-        static int last_val[NUMBER_OF_DICE] = { 0 } ;
-        int new_val = d->val - 1;
-        int dicebox = d - DiceValues;
-        
-        if (d->sel)
-                new_val = DIE_SELECTED_PIXMAP;
-        
-        if (last_val[dicebox]!=new_val) {
-                
-                gtk_widget_hide (dicePixmaps[dicebox][last_val[dicebox]]);
-                gtk_widget_show (dicePixmaps[dicebox][new_val]);
-                
-                last_val[dicebox] = new_val;
-        }
-        
-}
-
-
 void
-UpdateDie (int no)
+UpdateAllDicePixmaps (void)
 {
-        /* printf("Update die #%d\n",no); */
+        static int last_val[NUMBER_OF_DICE] = { 0 };
+        int i;
+        
+        for (i = 0; i < NUMBER_OF_DICE; i++) {
+                gtk_widget_hide (dicePixmaps[i][last_val[i]]);
+                last_val[i] = DiceValues[i].val - 1;
+                gtk_widget_show (dicePixmaps[i][last_val[i]]);
+        }
+}
 
-        UpdateDiePixmap (&DiceValues[no]);
+void
+DeselectAllDice(void)
+{
+	int i;
+
+	for (i = 0; i < NUMBER_OF_DICE; i++)
+		gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (diceBox[i]), FALSE);
 }
 
 /* Callback on dice press */
 gint 
-gnome_modify_dice (GtkWidget *widget, GdkEventButton *event, gpointer data) 
+gnome_modify_dice (GtkWidget *widget, gpointer data) 
 {
         DiceInfo *tmp = (DiceInfo *) data;
-
-        /* Ignore the 2BUTTON and 3BUTTON events. */
-        if (event && (event->type != GDK_BUTTON_PRESS))
-                return TRUE;
-
-        /* printf("Pressed die: %d. NumberOfRolls=%d\n",
-           tmp - DiceValues + 1, NumberOfRolls); */
-
-        /* Stop play when player is marked finished */
-	if (players[CurrentPlayer].finished)
+	GtkToggleToolButton *button = GTK_TOGGLE_TOOL_BUTTON(widget);
+ 
+        /* Don't modify dice if player is marked finished or computer is playing */
+	if (players[CurrentPlayer].finished || players[CurrentPlayer].comp) {
+		if (gtk_toggle_tool_button_get_active (button))
+			gtk_toggle_tool_button_set_active (button, FALSE);
                 return TRUE;
+	}
 
         if (NumberOfRolls >= NUM_ROLLS) {
-                say(_("You're only allowed three rolls! Select a score box."));
+                say(_("You are only allowed three rolls. Choose a score slot."));
+		gtk_toggle_tool_button_set_active (button, FALSE);
                 return TRUE;
         }
 
-#ifdef DEBUG
-	if (event->state & GDK_SHIFT_MASK) {
-                /* If the user is holding down shift, cycle through the dice
-                 * --very useful for debugging. */
-                tmp->val = (tmp->val + 1)%7;
-                if (! tmp->val)
-                        tmp->val++;
-
-                printf ("val before update: %d\n", tmp->val);
-                UpdateDiePixmap (tmp);
-        }
-	else
-#endif
-        {
-                /* Stop play when player is marked finished */
-                if (players[CurrentPlayer].finished)
-                        return TRUE;
-
-                if (NumberOfRolls >= NUM_ROLLS) {
-                        say (_("You're only allowed three rolls! Select a score box."));
-                        return TRUE;
-                }
-	    
-                tmp->sel = 1 - tmp->sel;
-
-                UpdateDiePixmap (tmp);
-        }
+        tmp->sel = gtk_toggle_tool_button_get_active (button);
 
+	update_roll_button_sensitivity();
 	return TRUE;
 }
 
@@ -379,67 +393,41 @@ say (char *fmt, ...)
 {
 	va_list ap;
 	char buf[200];
+	guint context_id;
 
 	va_start (ap, fmt);
 	g_vsnprintf (buf, 200, fmt, ap);
 	va_end (ap);
 
-	gnome_appbar_pop (GNOME_APPBAR (appbar));
-	gnome_appbar_push (GNOME_APPBAR (appbar), buf);
+	context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "message");
+	gtk_statusbar_pop (GTK_STATUSBAR (statusbar), context_id);
+	gtk_statusbar_push (GTK_STATUSBAR (statusbar), context_id, buf);
 }
 
 
 static gint
-about (GtkWidget *widget, gpointer data)
+about_cb (GtkAction *action, gpointer data)
 {
-	static GtkWidget *about = NULL;
         const gchar *authors[] = {
                 N_("GNOME version (1998):"),
 		"Scott Heavner",
-		N_("Curses version (1992):"),
+		"",
+		N_("Console version (1992):"),
 		"Orest Zborowski",
 		NULL
 	};
-        gchar *documenters[] = {
-                NULL
-        };
-        /* Translator credits */
-        gchar *translator_credits = _("translator-credits");
-	GdkPixbuf *pixbuf = NULL;
         
-
-	if (about != NULL) {
-	  	gtk_window_present (GTK_WINDOW (about));
-		return FALSE;
-	}
-        {
-		int i = 0;
-		while (authors[i] != NULL) {
-                        authors[i]=_(authors[i]); 
-                        i++;
-                }
-	}
-
-	pixbuf = gdk_pixbuf_new_from_file (GNOMEPIXMAPDIR "/gnome-gtali.png",
-                                           NULL);
-
-        about = gnome_about_new (appName, VERSION,
-				 "Copyright \xc2\xa9 1998-2004 Free Software "
-				 "Foundation, Inc.",
-				 _("A sort of poker with dice and less money."),
-				 (const char **)authors,
-				 (const char **)documenters,
-				 strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
-				 pixbuf);
-		
-        if (pixbuf != NULL)
-                gdk_pixbuf_unref (pixbuf);
-		  
-        gtk_window_set_transient_for (GTK_WINDOW (about),
-                                      GTK_WINDOW (window));
-	g_signal_connect (G_OBJECT (about), "destroy",
-                          G_CALLBACK (gtk_widget_destroyed), &about);
-        gtk_widget_show (about);
+	gtk_show_about_dialog (GTK_WINDOW (window),
+			       "name", appName,
+			       "version", VERSION,
+			       "copyright", "Copyright \xc2\xa9 1998-2005 "
+					    "Free Software Foundation, Inc.",
+			       "comments", _("A variation on poker with "
+					     "dice and less money."),
+			       "authors", authors,
+			       "translator_credits", _("translator-credits"),
+			       NULL);
+	
 	return FALSE;
 }
 
@@ -453,85 +441,33 @@ ShowHighScores (void)
 		gtk_window_set_transient_for (GTK_WINDOW (dialog),
                                               GTK_WINDOW (window));
 		gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+		gtk_dialog_set_has_separator (GTK_DIALOG(dialog), FALSE);
+		gtk_container_set_border_width (GTK_CONTAINER(dialog), 5);
+		gtk_box_set_spacing (GTK_BOX (GTK_DIALOG(dialog)->vbox), 2);
+		gtk_window_set_resizable (GTK_WINDOW(dialog), FALSE);
 	}
 }
 
 static gint 
-score_callback(GtkWidget *widget, gpointer data)
+score_callback(GtkAction *action, gpointer data)
 {
         ShowHighScores ();
 	return FALSE;
 }
 
-static gint 
-undo_callback(GtkWidget *widget, gpointer data)
-{
-        int i;
-	if (!GameIsOver ()) {
-                for (i=0; i < NumberOfPlayers; i++)
-                        ExecSingleUndo (SCORE_OK);
-                say (_("Cheater! Any high scores will not be recorded."));
-                undo_set_sensitive (UndoPossible ());
-        }
-	return FALSE;
-}
-
-/* Define menus later so we don't have to proto callbacks? */
-GnomeUIInfo gamemenu[] = {
-        GNOMEUIINFO_MENU_NEW_GAME_ITEM (new_game_callback, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_UNDO_MOVE_ITEM (undo_callback, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_SCORES_ITEM (score_callback, NULL),
-	GNOMEUIINFO_SEPARATOR,
-        GNOMEUIINFO_MENU_QUIT_ITEM (quit_game, NULL),
-	GNOMEUIINFO_END
-};
-
-GnomeUIInfo settingsmenu[] = {
-	GNOMEUIINFO_MENU_PREFERENCES_ITEM (setup_game, NULL),
-	GNOMEUIINFO_END
-};
-
-GnomeUIInfo helpmenu[] = {
-        GNOMEUIINFO_HELP ("gtali"),
-	GNOMEUIINFO_MENU_ABOUT_ITEM (about, NULL),
-	GNOMEUIINFO_END
-};
-
-GnomeUIInfo mainmenu[] = {
-	GNOMEUIINFO_MENU_GAME_TREE (gamemenu),
-	GNOMEUIINFO_MENU_SETTINGS_TREE (settingsmenu),
-	GNOMEUIINFO_MENU_HELP_TREE (helpmenu),
-	GNOMEUIINFO_END
-};
-
-static void undo_set_sensitive (gboolean state)
-{
-        gtk_widget_set_sensitive (gamemenu[2].widget, state);
-}
-
 static void
 LoadDicePixmaps(void)
 {
-        GtkWidget *tmp;
-	GdkPixbuf *pixbuf;
 	int i, j;
 
 	for (i=0; i < NUMBER_OF_PIXMAPS; i++) {
-		tmp = NULL;
-
-		/* Check for files w/pixmaps, override compiled defaults */
-		if (g_file_test (dicefiles[i], G_FILE_TEST_EXISTS))
-			tmp = gtk_image_new_from_file (dicefiles[i]);
-
-                dicePixmaps[0][i] = tmp;
-
-		for (j=0; j < NUMBER_OF_DICE; j++) {
-			pixbuf = gtk_image_get_pixbuf 
-                                (GTK_IMAGE (dicePixmaps[0][i]));
-			dicePixmaps[j][i] = gtk_image_new_from_pixbuf (pixbuf);
-		}
+                /* This is not efficient, but it lets us load animated types,
+                 * there is no way for us to copy a general GtkImage (the old 
+                 * code had a way for static images). */
+		if (g_file_test (dicefiles[i], G_FILE_TEST_EXISTS)) 
+			for (j=0; j < NUMBER_OF_DICE; j++)
+                                dicePixmaps[j][i] = gtk_image_new_from_file (dicefiles[i]);
+                /* FIXME: What happens if the file isn't found. */
 	}
 }
 
@@ -545,98 +481,173 @@ void update_score_state (void)
 	top = gnome_score_get_notable (appID, NULL,
                                        &names, &scores, &scoretimes);
 	if (top > 0) {
-		gtk_widget_set_sensitive (gamemenu[4].widget, TRUE);
+		gtk_action_set_sensitive (scores_action, TRUE);
 		g_strfreev (names);
 		g_free (scores);
 		g_free (scoretimes);
 	} else {
-		gtk_widget_set_sensitive (gamemenu[4].widget, FALSE);
+		gtk_action_set_sensitive (scores_action, FALSE);
 	}
 }
 
+static void
+help_cb (GtkAction *action, gpointer data)
+{
+        gnome_help_display ("gtali.xml", NULL, NULL);
+}
+
+
+static const GtkActionEntry action_entry[] = {
+	{ "GameMenu", NULL, N_("_Game") },
+	{ "SettingsMenu", NULL, N_("_Settings") },
+	{ "HelpMenu", NULL, N_("_Help") },
+	{ "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, G_CALLBACK (new_game_callback) },
+	{ "Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL, G_CALLBACK (score_callback) },
+	{ "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_game) },
+	{ "Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL, G_CALLBACK (setup_game) },
+	{ "Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL, G_CALLBACK (help_cb) },
+	{ "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (about_cb) },
+	/* Roll is just an accelerator */
+	{ "Roll", GTK_STOCK_REFRESH, NULL, "r", NULL, G_CALLBACK (gnome_roll_dice) }
+};
+
+
+static const char ui_description[] =
+"<ui>"
+"  <menubar name='MainMenu'>"
+"    <menu action='GameMenu'>"
+"      <menuitem action='NewGame'/>"
+"      <menuitem action='Scores'/>"
+"      <menuitem action='Quit'/>"
+"    </menu>"
+"    <menu action='SettingsMenu'>"
+"      <menuitem action='Preferences'/>"
+"    </menu>"
+"    <menu action='HelpMenu'>"
+"      <menuitem action='Contents'/>"
+"      <menuitem action='About'/>"
+"    </menu>"
+"  </menubar>"
+"  <accelerator action='Roll' />"
+"</ui>";
+
+
+static void
+create_menus (GtkUIManager *ui_manager)
+{
+	GtkActionGroup *action_group;
+
+	action_group = gtk_action_group_new ("group");
+
+	gtk_action_group_set_translation_domain(action_group, GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (action_group, action_entry, G_N_ELEMENTS (action_entry), window);
+
+	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+	gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
+	scores_action = gtk_action_group_get_action (action_group, "Scores");
+}
+
+
 static void
 GyahtzeeCreateMainWindow(void)
 {
-        GtkWidget *all_boxes;
-	GtkWidget *mbutton;
+        GtkWidget *hbox, *vbox;
+        GtkWidget *toolbar;
 	GtkWidget *tmp;
+        GtkWidget *dicebox;
+	GtkWidget *menubar;
+	GtkAccelGroup *accel_group;
+	GtkUIManager *ui_manager;
 	int i, j;
 
-        gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
-        g_signal_connect (G_OBJECT (window), "delete_event",
+	games_stock_init();
+        
+	g_signal_connect (G_OBJECT (window), "delete_event",
                           G_CALLBACK (quit_game), NULL);
         g_signal_connect (G_OBJECT (window), "key_press_event",
                           G_CALLBACK (key_press), NULL);
 
+	statusbar = gtk_statusbar_new ();
+	ui_manager = gtk_ui_manager_new ();
+
+	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbar), FALSE);
+	games_stock_prepare_for_statusbar_tooltips (ui_manager, statusbar);
+
 	/*---- Menus ----*/
-	gnome_app_create_menus (GNOME_APP (window), mainmenu);
-
-
-	/*---- Status Bar ----*/
-	appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
-	gnome_app_set_statusbar (GNOME_APP (window), appbar);
-	gnome_appbar_push (GNOME_APPBAR (appbar),
-			   _("Select dice to re-roll, press Roll!,"
-                             " or select score slot."));
-
-	gnome_app_install_menu_hints (GNOME_APP (window), mainmenu);
-
+	create_menus (ui_manager);
+	accel_group = gtk_ui_manager_get_accel_group (ui_manager);
+	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
+	menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
+	
 	/*---- Content ----*/
-        all_boxes = gtk_hbox_new (FALSE, 0);
-	gnome_app_set_contents (GNOME_APP (window), all_boxes);
 
+	hbox = gtk_hbox_new (FALSE, 0);
+	vbox = gtk_vbox_new (FALSE, 0);
+
+	gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), statusbar, FALSE, FALSE, 0);
+
+	gnome_app_set_contents (GNOME_APP (window), vbox);
+
+	gtk_widget_show (statusbar);
         /* Retreive dice pixmaps from memory or files */
         LoadDicePixmaps ();
         
 	/* Put all the dice in a vertical column */
-        diceTable = gtk_table_new (NUMBER_OF_DICE + 2, 1, FALSE);
-        gtk_box_pack_start(GTK_BOX(all_boxes), diceTable, FALSE, TRUE, 0);
+        dicebox = gtk_vbox_new (FALSE, 0);
+        gtk_box_pack_start (GTK_BOX(hbox), dicebox, FALSE, TRUE, 0);
+        gtk_widget_show (dicebox);
+
         rollLabel = gtk_label_new(NULL);
         gtk_label_set_use_markup(GTK_LABEL(rollLabel), TRUE);
-        gtk_table_attach(GTK_TABLE(diceTable), rollLabel, 0, 1, 0, 1,
-                         0, 0, 5, 10);
         gtk_widget_show(rollLabel);
+        gtk_box_pack_start (GTK_BOX (dicebox), rollLabel, FALSE, TRUE, 5);
+
+        mbutton = gtk_button_new_with_label (_("Roll!"));
+        gtk_box_pack_end (GTK_BOX (dicebox), mbutton, FALSE, FALSE, 5); 
+        g_signal_connect (G_OBJECT (mbutton), "clicked",
+                          G_CALLBACK (gnome_roll_dice), NULL);
+        gtk_widget_show (GTK_WIDGET (mbutton));
+        
+        toolbar = gtk_toolbar_new ();
+        gtk_toolbar_set_orientation (GTK_TOOLBAR (toolbar), 
+                                     GTK_ORIENTATION_VERTICAL);
+        gtk_toolbar_set_style (GTK_TOOLBAR (toolbar),
+                               GTK_TOOLBAR_ICONS);
+        gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), FALSE);
+        gtk_box_pack_end (GTK_BOX (dicebox), toolbar, TRUE, TRUE, 0); 
 
  	for (i=0; i < NUMBER_OF_DICE; i++) {
-
-                diceBox[i] = gtk_event_box_new ();
-                gtk_table_attach (GTK_TABLE (diceTable), diceBox[i], 0, 1,
-                                  i+1, i+2, 0, 0, 5, 5);
                 tmp = gtk_vbox_new(FALSE, 0);
-                gtk_container_add (GTK_CONTAINER (diceBox[i]),
-                                   tmp);
                 
                 for (j=0; j < NUMBER_OF_PIXMAPS; j++)
                         gtk_box_pack_start (GTK_BOX (tmp), 
                                             dicePixmaps[i][j],
                                             FALSE, FALSE, 0);
-                
-                gtk_widget_add_events (diceBox[i],
-                                       GDK_BUTTON_PRESS_MASK);
-                
-                g_signal_connect (G_OBJECT (diceBox[i]), "button_press_event",
+
+                diceBox[i] = gtk_toggle_tool_button_new ();
+                gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON(diceBox[i]), tmp);
+                g_signal_connect (G_OBJECT (diceBox[i]), "clicked",
                                   G_CALLBACK (gnome_modify_dice),
                                   &DiceValues[i]);
 
+                gtk_toolbar_insert (GTK_TOOLBAR (toolbar),
+                                    GTK_TOOL_ITEM (diceBox[i]), -1);
+
+                gtk_widget_show (GTK_WIDGET (diceBox[i])); 
+                gtk_widget_show (tmp); 
                 gtk_widget_show (dicePixmaps[i][0]);
-                gtk_widget_show (tmp);
-                gtk_widget_show (diceBox[i]);
 	}
-	mbutton = gtk_button_new_with_label (_("Roll!"));
-	gtk_table_attach (GTK_TABLE (diceTable), mbutton, 0, 1, i+1, i+2,
-                          0, 0, 5, 5);
-        g_signal_connect (G_OBJECT (mbutton), "clicked",
-                          G_CALLBACK (gnome_roll_dice), NULL);
-	gtk_widget_show (mbutton);
-	gtk_widget_show (diceTable);
+        gtk_widget_show (toolbar);
 
 	/* Scores displayed in score list */
         ScoreList = create_score_list ();
-        gtk_box_pack_end (GTK_BOX (all_boxes), ScoreList, TRUE, TRUE, 0);
+        gtk_box_pack_end (GTK_BOX (hbox), ScoreList, TRUE, TRUE, 0);
         setup_score_list (ScoreList);
         gtk_widget_show (ScoreList);
         
-	gtk_widget_show (all_boxes);
+	gtk_widget_show (hbox);
 
         gtk_widget_show (window);
 
@@ -653,7 +664,6 @@ main (int argc, char *argv[])
         gint i;
 
 	gnome_score_init (appID);
-
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
@@ -704,7 +714,6 @@ main (int argc, char *argv[])
         DisplayComputerThoughts = gconf_client_get_bool (client,
                                                          "/apps/gtali/DisplayComputerThoughts",
                                                          NULL);
-        
         /* Read in new player names */
         name_list = gconf_client_get_list (client, "/apps/gtali/PlayerNames",
                                            GCONF_VALUE_STRING, &err);
